package org.baderlab.csplugins.enrichmentmap;

/**
 * Created by
 * User: risserlin
 * Date: Oct 14, 2009
 * Time: 10:57:32 AM
 * <p>
 * Parent class to generic and GSEA results
 */
public class EnrichmentResult {

    /**
     * Default Class constructor
     */
    public EnrichmentResult() {
    }


}
